###################################################################################################
# _exportable.py
#
# Implementation of class Exportable (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import string
import urllib
import tempfile
import os
from OFS.Image import Image
# Product Imports.
from _globals import *
import _ftpmanager
import _fileutil
import _xmllib

###################################################################################################
# exportCommon:
###################################################################################################
def exportCommon(root, path, id):
  if hasattr(root,id):
    folder = getattr(root,id)
    if folder.meta_type == 'Folder':
      for ob in folder.objectValues():
        if ob.meta_type == 'Folder':
          ob_id = ob.id
          exportCommon(ob,'%s/%s'%(path,id),ob_id)
        else:
          ob_id = ob.id()
          print "[%s]: Export %s..."%(path,ob_id)
          _fileutil.exportObj(ob,'%s/%s/%s'%(path,id,ob_id))

###################################################################################################
# findDelimiter:
###################################################################################################
def findDelimiter(s, delimiters=['"',"'"]):
  rtn = -1
  for delimiter in delimiters:
    i = string.find(s,delimiter)
    if rtn == -1:
      rtn = i
    elif i >= 0:
      rtn = min(rtn,i)
  return rtn

###################################################################################################
# rfindDelimiter:
###################################################################################################
def rfindDelimiter(s, delimiters=['"',"'"]):
  rtn = -1
  for delimiter in delimiters:
    i = string.rfind(s,delimiter)
    rtn = max(rtn,i)
  return rtn

###################################################################################################
# localHtml:
###################################################################################################
def localHtml(self, obj, html, REQUEST):
  
  # Process relative links to Blob-Fields (Images and Files).
  # ---------------------------------------------------------
  
  # [i]
  for s_blob in ['blob_html?','f_zoomImage?']:
    i = string.find(html,s_blob)
    while i > 0:
      k = rfindDelimiter(html[:i])
      j = string.rfind(html[k:i-1],'/')
      if j < 0:
        s_blob_id = obj.id
        o_blob = obj
        s_new = '"'
      else:
        s_blob_id = html[k+j+1:i-1]
        o_blob = self.findObjId(s_blob_id,REQUEST)
        s_new = '%s/'%s_blob_id
      s_old = s_new + s_blob
      if o_blob is not None:
        l = string.find(html[i:],'key=')
        m = string.find(html[i+l:],'&')
        s_key = html[i+l+4:i+l+m]
        value = o_blob.getObjProperty(s_key,REQUEST)
        filename = getattr(value,'filename',None)
        s_new = s_new + filename
      s_new = s_new + "?"
      if len(s_old) > 0: html = string.replace(html,s_old,s_new)
      print '[localHtml]: [i] %s -> %s'%(s_old,s_new)
      i = string.find(html,s_blob)
  
  # [ii]
  s_blob = '?key='
  i = string.find(html,s_blob)
  while i > 0:
    k = findDelimiter(html[i:])
    s_old = html[i:i+k]
    s_new = ''
    print '[localHtml]: [ii] %s -> %s'%(s_old,s_new)
    if len(s_old) > 0: html = string.replace(html,s_old,s_new)
    i = string.find(html,s_blob)
  
  # Process relative links to JavaScript.
  # -------------------------------------
  for js in ['comlib','formlib']:
    s_old = '%s_js?lang=%s"'%(js,REQUEST['lang'])
    s_new = '%s.js"'%(js)
    if len(s_old) > 0: html = string.replace(html,s_old,s_new)
  
  return html


###################################################################################################
# localPrintHtml:
###################################################################################################
def localPrintHtml(self, obj, html, REQUEST):
  
  # Process absolute URLs.
  s_old = '"%s/'%self.getDocumentElement().absolute_url()
  s_new = '"'
  if len(s_old) > 0: html = string.replace(html,s_old,s_new)
  
  # Process relative links to Child-Objects.
  for child in obj.getChildNodes(REQUEST):
    s_old = '"%s'%child.id
    s_new = '"./%s'%child.relative_obj_path()
    if len(s_old) > 0: html = string.replace(html,s_old,s_new)
  
  # Remove styles.
  startStyle = "<style"
  endStyle = "</style>"
  i = string.find(html,startStyle)
  j = string.find(html,endStyle)
  while i >= 0 and  j > i:
    html = html[0:i] + html[j+len(endStyle):]
    i = string.find(html,startStyle)
    j = string.find(html,endStyle)
  
  return html


###################################################################################################
# localIndexHtml:
###################################################################################################
def localIndexHtml(self, obj, html, REQUEST):
   
   # Parse HTML.
   # -----------
   sRoot = ''
   for i in range(0,obj.getLevel()):
     sRoot = '../%s'%sRoot
   
   # Process absolute URLs (I)
   s_old = '%s/'%self.getDocumentElement().absolute_url()
   s_new = '%s'%sRoot
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)
   
   # Process absolute URLs (II)
   s_old = '%s/'%self.getDocumentElement().aq_parent.absolute_url()
   s_new = '%s'%sRoot
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)
   
   # Process links to common-Folder (resource-Directory: Images and Assets).
   s_old = '"./common/'
   s_new = '"%scommon/'%sRoot
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)
   s_old = '"common/'
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)
   
   # Process links to Product-Folder (resource-Directory: Images and Assets).
   s_old = '"misc_/zms/'
   s_new = '"%smisc_/zms/'%sRoot
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)

   # Process preview parameters.
   s_old = '?preview=preview'
   s_new = ''
   html = string.replace(html,s_old,s_new)
   #++ print '[localIndexHtml]: %s -> %s'%(s_old,s_new)

   return html


###################################################################################################
###################################################################################################
###
###   class Exportable
###
###################################################################################################
###################################################################################################
class Exportable:

    ###############################################################################################    
    #  Exportable.manage_export:
    #
    #  Exports ZMS-object.
    ###############################################################################################    
    def manage_export(self, export_format, lang, manage_lang, REQUEST, RESPONSE):
        """ Exportable.manage_export """
        #++ print "[%s.manage_export]: "%(self.meta_type)
        
        title = self.getDocumentElement().aq_parent.id
        
        # ZEXP
        if export_format == 0:
          filename = '%s.zexp'%self.id
          export = self.aq_parent.manage_exportObject(id=self.id, download=1)
          content_type = 'application/data'
          
        # HTML
        elif export_format == 1:
          filename = '%s_html.zip'%title
          export = self.toZippedHtml(REQUEST)
          content_type = 'application/zip'
        
        # XML
        elif export_format == 2: 
          filename = '%s_%s_xml.zip'%(title,self.meta_type)
          export = self.toZippedXml(REQUEST)
          content_type = 'application/zip'
        
        # FTP
        elif export_format == 3: 
          export = _ftpmanager.ftpToProvider(self,export_format,lang,manage_lang,REQUEST,RESPONSE)
          return export
        
        # HTML (printable)
        elif export_format == 4: 
          filename = '%s.html'%title
          export = self.toPrintableHtml(REQUEST)
          content_type = 'text/html'
        
        # HTML (zipped printable)
        elif export_format == 7: 
          filename = '%s_html.zip'%title
          export = self.toZippedPrintableHtml(REQUEST)
          content_type = 'application/zip'
        
        # RTF (printable)
        elif export_format == 5: 
          filename = '%s_rtf.zip'%title
          export = self.toPrintableRtf(REQUEST)
          content_type = 'application/zip'
        
        # PDF (printable)
        elif export_format == 6: 
          filename = '%s_pdf.zip'%title
          export = self.toPrintablePdf(REQUEST)
          content_type = 'application/zip'
        
        # return export for download to browser
        RESPONSE.setHeader('Content-Type',content_type)
        RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
        return export

    # ---------------------------------------------------------------------------------------------
    #	Exportable.__prepare_temp_export_folder__:
    #
    #	Return path with resources (Images, StyleSheets, etc.)
    # ---------------------------------------------------------------------------------------------
    def __prepare_temp_export_folder__(self, from_content=1, from_zms=0, from_common=0, REQUEST=None):
        
        # --- Create temporary folder.
        tempfolder = tempfile.mktemp()
        
        # Copy resource files.
        # --------------------
        
        # from ZMS.
        if from_zms:
          folder = 'misc_/zms'
          for obj_id in self.misc_.zms._d.keys():
            print "[%s]: Export %s..."%(folder,obj_id)
            _fileutil.exportObj(self.misc_.zms[obj_id],'%s/%s/%s'%(tempfolder,folder,obj_id))
        
        # from common directory (template-specific: Grafiken und Assets).
        if from_common:
          exportCommon(self.getDocumentElement().aq_parent,tempfolder,'common')
        
        # from content.
        if from_content:
          
          if REQUEST is not None:
          
            # Save CStyleSheet to file.
            # -------------------------
            filename = "%s.css"%self.getDocumentElement().aq_parent.id
            data = self.getStylesheet().index_html(REQUEST,RESPONSE)
            _fileutil.exportObj(data,"%s/%s"%(tempfolder,filename))
          
            # Save JavaScript to file.
            # ------------------------
            for js in ['comlib','formlib']:
              filename = '%s.js'%(js)
              data = getattr(self,'%s_js'%(js))(self,REQUEST)
              _fileutil.exportObj(data,"%s/%s"%(tempfolder,filename))
          
          # Download ressources (images & files)
          self.recurse_downloadRessources(tempfolder)
        
        return tempfolder


    # ---------------------------------------------------------------------------------------------
    #	Exportable.recurse_downloadHtmlPages:
    # ---------------------------------------------------------------------------------------------
    def recurse_downloadHtmlPages(self, obj, sFolder, REQUEST):
        
        if len(obj.getChildNodes()) > 0:
           
           try:
             os.mkdir(sFolder)
           except:
             pass
           
           for s_lang in self.getLanguages():
             
             REQUEST.set('lang',s_lang)
             REQUEST.set('preview',None)
             
             # Get index_html.
             # --------------
             dctOp = {'index':'','sitemap':'sitemap','index_print':'print'}
             for s_id in dctOp.keys():
             
               s_index_html = '%s/%s_%s.html'%(sFolder,s_id,s_lang)
               print '[recurse_downloadHtmlPages]: %s(%i)'%(s_index_html,obj.getLevel())
               REQUEST.set('op',dctOp[s_id])
               html = obj.index_html(obj,REQUEST)
               html = localHtml(self,obj,html,REQUEST)
               html = localIndexHtml(self,obj,html,REQUEST)
               
               # Save index_html to file.
               f = open(s_index_html,'w')
               f.write(html)
               f.close()
               
               # Root folder requires and defaults to "index.html" at most systems.
               if s_id == 'index' and \
                  s_lang == self.getPrimaryLanguage() and \
                  obj.getLevel() == 0:
                 s_index_html = '%s/index.html'%(sFolder)
                 f = open(s_index_html,'w')
                 f.write(html)
                 f.close()
           
           # Process children.
           for oChild in obj.getChildNodes():
             self.recurse_downloadHtmlPages(oChild,'%s/%s'%(sFolder,oChild.id),REQUEST)


    # ---------------------------------------------------------------------------------------------
    #	Exportable.toZippedHtml:
    # ---------------------------------------------------------------------------------------------
    def toZippedHtml(self, REQUEST):
      #++ print "[%s.toZippedHtml]: "%(self.meta_type)
      
      # Create temporary folder.
      tempfolder = self.__prepare_temp_export_folder__(from_content=1,from_zms=1,from_common=1,REQUEST=REQUEST)
      
      # Download HTML-pages.
      self.recurse_downloadHtmlPages(self,tempfolder,REQUEST)
      
      # Get zip-file.
      rtn = _fileutil.buildZipArchive('%s/*.*'%tempfolder)
      
      # Remove temporary folder.
      _fileutil.remove(tempfolder,deep=1)
      
      return rtn


    # ---------------------------------------------------------------------------------------------
    #  Exportable.toZippedXml:
    # ---------------------------------------------------------------------------------------------
    def toZippedXml(self, REQUEST):
      #++ print '[%s.toZippedXml]: '%(self.meta_type)
      
      # Create temporary folder.
      tempfolder = self.__prepare_temp_export_folder__()
      
      # Get xml-export.
      myDebugger.open()
      sXml ='<?xml version="1.0"?>' + _xmllib.getObjToXml(self)
      
      # Write xml-export to file.
      sTmpXmlFilename = _fileutil.getOSPath('%s/%s_%s.xml'%(tempfolder,self.getDocumentElement().aq_parent.id,self.meta_type))
      _fileutil.exportObj(sXml,sTmpXmlFilename)
      
      # Get zip-file.
      sZipFiles = _fileutil.getOSPath('%s/*.*'%tempfolder)
      rtn = _fileutil.buildZipArchive(sZipFiles)
      
      # Remove temporary folder.
      _fileutil.remove(tempfolder,deep=1)
      
      return rtn

###################################################################################################
