###################################################################################################
# zmscustom.py
#
# Implementation of class ZMSCustom (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
import string 
import urllib
import time
import sys
# Product Imports.
from zmscontainerobject import ZMSContainerObject
from zmsobject import ZMSObject
import _globals


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSCustomForm = HTMLFile('manage_addzmscustomform', globals()) 
def manage_addZMSCustom(self, meta_id, meta_name, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSCustom """
  
  if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
    
    ##### Create ####
    obj = ZMSCustom(self.getNewId(),_sort_id+1,meta_id)
    self._setObject(obj.id, obj)
  
    for oItem in self.objectValues():
      if oItem.id == obj.id:
        ##### Object State ####
        oItem.setObjStateNew(REQUEST)
        ##### Init Properties ####
        oItem.manage_changeProperties(lang,manage_lang,REQUEST,RESPONSE)
        ##### VersionManager ####
        oItem.__autoCommit__(REQUEST)
  
    ##### Normalize Sort-IDs ####
    self.normalizeSortIds()
  
    # Return with message.        
    message = self.getLangStr('MSG_INSERTED',manage_lang)%meta_name
    RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))

  else:
    RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s'%(self.absolute_url(),lang,manage_lang))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSCustom(ZMSContainerObject): 
        
    # Properties.
    # -----------
    meta_type = "ZMSCustom"
    icon = "misc_/zms/zmsdocument.gif"
    icon_disabled = "misc_/zms/zmsdocument_disabled.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',    'action': 'manage_main'},
	{'label': 'TAB_ACCESS',  'action': 'manage_users'},
	{'label': 'TAB_PREVIEW', 'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_properties','manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'LangDate','lang_inherit':_globals.LI_FORBIDDEN},
        # Versioned by
        'work_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'work_dt':{'datatype':'LangDate','lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'sort_id':{'datatype':'string','native':1},
        'meta_id':{'datatype':'string','native':1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_start':{'datatype':'LangDate','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_end':{'datatype':'LangDate','lang_inherit':_globals.LI_OBLIGATE},
    }


    # Management Interface.
    manage_main = HTMLFile('dtml/metaobj/manage_main', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSCustom.__init__: 
    #
    # Constructor (initialise a new instance of ZMSCustom).
    ###############################################################################################
    def __init__(self, id='', sort_id='', meta_id=''): 
      """ ZMSCustom.__init__ """
      ZMSObject.__init__(self,id,sort_id)
      self.meta_id = meta_id


    ###############################################################################################
    # ZMSCustom.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
      message = ''

      # Process super-objects.
      message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        
      # Process object.
      # [...]

      # ---------------------------------------------------------------------
      # 20.02.2002
      # Build #061: Make object-attributes multilingual.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '061':
        attrs = []
        for attr in self.getMetaobj(self.meta_id)['attrs']:
	  if attr['type'][:3] != 'ZMS':
            attrs.append(attr['id'])
        for key in attrs:
          if key in self.getObjAttrs().keys():
            for s_preview in [None,'preview']:
              req = {'preview':s_preview}
              ob = self.getObjVersion(req)
              if hasattr(ob,key):
                for s_lang in self.getLanguages():
                  attr = '%s_%s'%(key,s_lang)
                  value = getattr(ob,key)
                  #++ print "[%s.recurse_updateVersion]: %s.%s=%s"%(self.meta_type,str(ob),attr,str(value))
                  setattr(ob,attr,value)
	    self.__delObjProperty__(key)

      # ---------------------------------------------------------------------
      # 26.02.2002
      # Build #065: Correct bug in #064.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '064':
        ob_live = self.getObjVersion({'preview':None})
        ob_work = self.getObjVersion({'preview':'preview'})
        for key in attrs:
          for s_lang in self.getLanguages():
            attr = '%s_%s'%(key,s_lang)
            if hasattr(ob_live,attr):
              value = getattr(ob_live,attr)
              if not hasattr(ob_work,attr) or \
                 getattr(ob_work,attr) != value and len(self.getObjStates()) == 0:
                #++ print "[%s.recurse_updateVersion]: Rescue %s..."%(self.meta_type,attr)
                setattr(ob_work,attr,value)

      # ---------------------------------------------------------------------
      # 26.02.2002
      # Build #067: Activity-Toggle.
      # ---------------------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '067':
        for s_preview in [None,'preview']:
          req = {'preview':s_preview}
          ob = self.getObjVersion(req)
          for s_key in ['active']:
            for s_lang in self.getLanguages():
              attr = '%s_%s'%(s_key,s_lang)
              if not hasattr(ob,attr) or getattr(self,'build','') <= '067':
                active = getattr(ob,'attr_active_start_%s'%s_lang,None) is not None or \
                         getattr(ob,'attr_active_end_%s'%s_lang,None) is not None 
  	        #++ print "[%s.recurse_updateVersion]: Activity-Toggle %s.%s=%i"%(self.meta_type,str(ob),attr,active)                       
                setattr(ob,attr,active)

      # Recurse through child-nodes.
      for ob in self.getChildNodes():
        message = message + ob.recurse_updateVersion(lang,manage_lang,REQUEST)
      
      # Return with message.
      return message

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getObjChildren
    # ---------------------------------------------------------------------------------------------
    def getObjChildren(self, id, REQUEST):
      obs = []
      for ob in self.getChildNodes(REQUEST):
        if len(id)==0 or ob.id[:len(id)]==id:
          obs.append(ob)
      return obs

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.initObjChild
    # ---------------------------------------------------------------------------------------------
    def initObjChild(self, id, _sort_id, type, REQUEST):

      ##### Create ####
      obj = self.dGlobalAttrs[type]['obj_class'](id,_sort_id+1)
      self._setObject(obj.id, obj)

      for oItem in self.objectValues():
        if oItem.id == obj.id:
          ##### Object State ####
          oItem.setObjStateNew(REQUEST)
          ##### Init Properties ####
          for s_lang in self.getLanguages():
            oItem.setObjProperty('active',1,s_lang)
          ##### VersionManager ####
          oItem.__autoCommit__(REQUEST)
          
      ##### Normalize Sort-IDs ####
      self.normalizeSortIds(_globals.id_prefix(id))
        
      return obj
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.initObjChildren
    # ---------------------------------------------------------------------------------------------
    def initObjChildren(self, REQUEST):
      for obj_attr in self.getMetaobj(self.meta_id)['attrs']:
        type = obj_attr['type']
        if type in self.dGlobalAttrs.keys():
          id = obj_attr['id']
          repetitive = obj_attr.has_key('repetitive') and obj_attr['repetitive']==1
          if repetitive:
            if id in self.objectIds():
              print "[%s.initObjChildren]: Rename %s to %s0"%(self.meta_type,id,id)
              self.manage_renameObject(id=id,new_id=id+'0')
          else:
            if id+'0' in self.objectIds():
              print "[%s.initObjChildren]: Rename %s0 to %s"%(self.meta_type,id,id)
              self.manage_renameObject(id=id+'0',new_id=id)
            elif not id in self.objectIds():
              print "[%s.initObjChildren]: Create %s"%(self.meta_type,id)
              self.initObjChild(id,0,type,REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.display_type
    # ---------------------------------------------------------------------------------------------
    def display_type(self, REQUEST=None, meta_type=''):
      if meta_type == '':
        try:
          ob = self.getMetaobj(self.meta_id)
          return ob['name']      
        except: 
          pass
      return ZMSObject.display_type(self,REQUEST,meta_type) # Inherit from Super-Class.


    """
    ###############################################################################################    
    #
    #   Eigenschaften
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSCustom.manage_changeProperties: 
    #
    #  Change Custom properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ ZMSCustom.manage_changeProperties """
        
        message = ''
        if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
          
          ##### DELETE OBJECT-CHILD ####
          if REQUEST['btn'] == 'delete':
            id = REQUEST['id']
            return self.manage_deleteObjs(lang,manage_lang,[id],REQUEST,RESPONSE)
          
          ##### CREATE OBJECT-CHILD ####
          elif REQUEST['btn'] == 'create':
            id = REQUEST['id'] + str(self.getSequence().nextVal())
            sort_id = self.getNewSortId()
            type = REQUEST['type']
            obj = self.initObjChild(id,sort_id,type,REQUEST)
            message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
            return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(obj.id,lang,manage_lang,urllib.quote(message)))
          
          ##### MOVE-UP OBJECT-CHILD ####
          elif REQUEST['btn'] == 'move_up':
            id = REQUEST['id']
            ob = getattr(self,id)
            return ob.manage_moveObjUp(lang,manage_lang,REQUEST,RESPONSE)
          
          ##### MOVE-DOWN OBJECT-CHILD ####
          elif REQUEST['btn'] == 'move_down':
            id = REQUEST['id']
            ob = getattr(self,id)
            return ob.manage_moveObjDown(lang,manage_lang,REQUEST,RESPONSE)
          
          else:
            
            ##### Object State #####
            self.setObjStateModified(REQUEST)
            
            ##### Properties #####
            for key in self.getObjAttrs().keys():
              if not key in ['change_uid','change_dt']:
                self.setReqProperty(key,REQUEST)
            
            ##### VersionManager ####
            self.__autoCommit__(REQUEST)
            
            ##### Message #####
            message = self.getLangStr('MSG_CHANGED',manage_lang)
          
          # Return with message.
          return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))
        
        # Return with message.
        self.checkIn(REQUEST)
        return RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(self.getParentNode().absolute_url(),lang,manage_lang,urllib.quote(message),self.id))


    # -------------------------------------------------------------------------------------------------
    #	ZMSCustom.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self): return self.getType() == 'ZMSDocument'

    # -------------------------------------------------------------------------------------------------
    #	ZMSObject.isPageElement:
    # -------------------------------------------------------------------------------------------------
    def isPageElement(self): return self.getType() == 'ZMSObject'

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getType
    # ---------------------------------------------------------------------------------------------
    def getType(self): return self.getMetaobj(self.meta_id)['type']

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getDtmlTemplate
    # ---------------------------------------------------------------------------------------------
    def getDtmlTemplate(self):
      root = self.getDocumentElement().aq_parent
      meta_type = self.getType()
      if meta_type == 'ZMSObject':
        tmplt = '%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSDocument':
        tmplt = 'bodyContent%s_%s'%(self.meta_type,self.meta_id)
      elif meta_type == 'ZMSTeaserElement':
        tmplt = 'bodyContentTeaser%s_%s'%(self.meta_type,self.meta_id)
      dtml_tmplt = getattr(root,tmplt,None)
      return dtml_tmplt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST): return self.getParentNode().getDCCoverage(REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST):
      s_titleAlt = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][0]
          s_titleAlt = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      else:
        try:
          for oAttr in self.getMetaobj(self.meta_id)['attrs']:
            if len(s_titleAlt) == 0:
              key = oAttr['id']
              datatype = oAttr['type']
              if datatype not in self.dGlobalAttrs.keys():
                value = self.getObjProperty(key,REQUEST)
                s_titleAlt = self.search_quote(self.getObjProperty(key,REQUEST),80,'')
        except:
          pass
      if len(s_titleAlt) == 0: 
        s_titleAlt = self.display_type(REQUEST)
      return s_titleAlt

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.getTitle
    # ---------------------------------------------------------------------------------------------
    def getTitle(self,REQUEST): 
      s_title = ""
      if self.isPage():
        try:
          attr = self.getDocumentElement().getMetaobj(self.meta_id)['attrs'][1]
          s_title = str(self.getObjProperty(attr['id'],REQUEST))
        except:
          pass
      if len(s_title) == 0: 
        s_title = self.display_type(REQUEST)
      return s_title


    """
    ###############################################################################################    
    ###  
    ###  HTML-Darstellung 
    ### 
    ###############################################################################################    
    """

    # preload display interface for custom-objects
    render_default = HTMLFile('dtml/metaobj/render_default', globals()) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom._getBodyContent:
    #
    #  HTML presentation of Special Document. 
    # ---------------------------------------------------------------------------------------------
    def _getBodyContent(self, REQUEST, thumbnail=0):
      s = ""
      try:
        dtml_tmplt = self.getDtmlTemplate()
        if self.getType() == 'ZMSObject':
          s = s + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSDocument':
          s = s + dtml_tmplt(self,REQUEST)
        elif self.getType() == 'ZMSTeaserElement':
          s = s + '<table cellspacing="0" cellpadding="0" border="0" width="100%" class="teaserElement">\n'
          s = s + '<tr'
          if REQUEST.has_key('bgcolor_text'):
            s = s + ' class="%s"'%str(REQUEST['bgcolor_text'])
          s = s + '>\n'
          s = s + '<td align="left" valign="top" class="teaserText">'
          s = s + '<p class="teaserText">%s</p>'%dtml_tmplt(self,REQUEST)
          s = s + '</td>\n'
          s = s + '</tr>\n'
          s = s + '</table>\n'
      except:
        s = s + '[Exception]:<br>'
        s = s + '-  Type: <strong>%s</strong><br>'%str(sys.exc_type)
        s = s + '-  Value: <strong>%s</strong><br>'%str(sys.exc_value)
        s = s + '-  Traceback: <strong>%s</strong>'%str(sys.exc_traceback)
      # Return <html>.
      return s

    # ---------------------------------------------------------------------------------------------
    #  ZMSCustom.renderShort:
    #
    #  Renders short presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def renderShort(self, REQUEST):
      html = self._getBodyContent(REQUEST)
      if len(html) == 0 or self.getType() == 'ZMSDocument':
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      # Return <html>.
      return html

    # ---------------------------------------------------------------------------------------------
    #	ZMSCustom.printHtml:
    #
    #	Renders print presentation of Special Document.
    # ---------------------------------------------------------------------------------------------
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = self._getBodyContent(REQUEST)
      if len(html) == 0: 
        html = self.render_default(self,ob=self,REQUEST=REQUEST)
      # Return <html>.
      return html

###################################################################################################
